<?php

require_once("Rest.inc.php");

	class API extends REST {

		public $data = "";
		const demo_version = false;

		private $db 	= NULL;
		private $mysqli = NULL;
		public function __construct() {
			// Init parent contructor
			parent::__construct();
			// Initiate Database connection
			$this->dbConnect();
		}

		/*
		 *  Connect to Database
		*/
		private function dbConnect() {
			require_once ("../../includes/config.php");
			$this->mysqli = new mysqli($host, $user, $pass, $database);
			$this->mysqli->query('SET CHARACTER SET utf8mb4');
		}

		/*
		 * Dynmically call the method based on the query string
		 */
		public function processApi() {
			if(isset($_REQUEST['x']) && $_REQUEST['x']!="") {
				$func = strtolower(trim(str_replace("/","", $_REQUEST['x'])));
				if((int)method_exists($this,$func) > 0) {
					$this->$func();
				} else {
					$respon = array('status' => 'failed');
					$this->response($this->json($respon), 200);
					exit;
				}
			} else {
				$respon = array('code' => '403', 'message' => 'processApi - method not exist');
				$this->response($this->json($respon), 200);
				exit;
			}
		}

		/* Api Checker */
		public function check_connection() {
			if (mysqli_ping($this->mysqli)) {
				//echo "Responses : Congratulations, database successfully connected.";
                $respon = array(
                    'status' => 'ok', 'database' => 'connected'
                );
                $this->response($this->json($respon), 200);
			} else {
                $respon = array(
                    'status' => 'failed', 'database' => 'not connected'
                );
                $this->response($this->json($respon), 404);
			}
		}

		public function categories() {

			if($this->get_request_method() != "GET") $this->response('',406);
			$limit = isset($this->_request['count']) ? ((int)$this->_request['count']) : 10;
			$page = isset($this->_request['page']) ? ((int)$this->_request['page']) : 1;

			$settings = $this->get_one_result("SELECT api_key FROM tbl_settings WHERE id = '1'");
			$api_key = implode(" ", $settings);

			$offset = ($page * $limit) - $limit;
			$count_total = $this->get_count_result("SELECT COUNT(DISTINCT cid) FROM tbl_category");

			//$sql = "SELECT cid AS 'category_id', category_name, category_image FROM tbl_category ORDER BY cid DESC LIMIT $limit OFFSET $offset";

			$sql = "SELECT DISTINCT c.cid AS 'category_id', c.category_name, c.category_image, COUNT(DISTINCT b.book_id) as book_count
					  FROM tbl_category c LEFT JOIN tbl_book b ON c.cid = b.category_id GROUP BY c.cid ORDER BY c.cid DESC LIMIT $limit OFFSET $offset";

			$posts = $this->get_list_result($sql);
			$count = count($posts);

			$respon = array(
				'status' => 'ok',
				'count' => $count,
				'count_total' => $count_total,
				'pages' => $page,
				'categories' => $posts
			);

			if (isset($_GET['api_key'])) {
				$key_received = $_GET['api_key'];
				if ($key_received == $api_key) {
					$this->response($this->json($respon), 200);
				} else {
					$this->failed();
				}
			} else {
				$this->failed();
			}

		}

		public function books() {

			if($this->get_request_method() != "GET") $this->response('',406);
			$limit = isset($this->_request['count']) ? ((int)$this->_request['count']) : 10;
			$page = isset($this->_request['page']) ? ((int)$this->_request['page']) : 1;

			$sort = isset($this->_request['sort']) ? ($this->_request['sort']) : 'book_id';
			$order = isset($this->_request['order']) ? ($this->_request['order']) : 'DESC';

			$settings = $this->get_one_result("SELECT api_key FROM tbl_settings WHERE id = '1'");
			$api_key = implode(" ", $settings);

			$offset = ($page * $limit) - $limit;
			$count_total = $this->get_count_result("SELECT COUNT(DISTINCT book_id) FROM tbl_book");

			$sql = "SELECT b.*, c.category_name FROM tbl_book b, tbl_category c WHERE b.category_id = c.cid ORDER BY $sort $order LIMIT $limit OFFSET $offset";

			$posts = $this->get_list_result($sql);
			$count = count($posts);

			$respon = array(
				'status' => 'ok',
				'count' => $count,
				'count_total' => $count_total,
				'pages' => $page,
				'books' => $posts
			);

			if (isset($_GET['api_key'])) {
				$key_received = $_GET['api_key'];
				if ($key_received == $api_key) {
					$this->response($this->json($respon), 200);
				} else {
					$this->failed();
				}
			} else {
				$this->failed();
			}

		}

		public function book_details() {

	    	$id = $_GET['id'];

			if($this->get_request_method() != "GET") $this->response('',406);

			$query_post = "SELECT b.*, c.category_name FROM tbl_book b, tbl_category c WHERE b.category_id = c.cid AND b.book_id = $id";

			$book = $this->get_one_result($query_post);
			
			$respon = array(
				'status' => 'ok', 'book' => $book
			);
			$this->response($this->json($respon), 200);

	    }		

		public function category_details() {

			if($this->get_request_method() != "GET") $this->response('',406);
			$limit = isset($this->_request['count']) ? ((int)$this->_request['count']) : 10;
			$page = isset($this->_request['page']) ? ((int)$this->_request['page']) : 1;
			$category_id = isset($this->_request['category_id']) ? ((int)$this->_request['category_id']) : 0;

			$sort = isset($this->_request['sort']) ? ($this->_request['sort']) : 'book_id';
			$order = isset($this->_request['order']) ? ($this->_request['order']) : 'DESC';

			$settings = $this->get_one_result("SELECT api_key FROM tbl_settings WHERE id = '1'");
			$api_key = implode(" ", $settings);

			$offset = ($page * $limit) - $limit;
			$count_total = $this->get_count_result("SELECT COUNT(DISTINCT book_id) FROM tbl_book WHERE category_id = $category_id");

			$sql = "SELECT b.*, c.category_name FROM tbl_book b, tbl_category c WHERE b.category_id = c.cid AND b.category_id = $category_id ORDER BY $sort $order LIMIT $limit OFFSET $offset";

			$posts = $this->get_list_result($sql);
			$count = count($posts);

			$respon = array(
				'status' => 'ok',
				'count' => $count,
				'count_total' => $count_total,
				'pages' => $page,
				'books' => $posts
			);

			if (isset($_GET['api_key'])) {
				$key_received = $_GET['api_key'];
				if ($key_received == $api_key) {
					$this->response($this->json($respon), 200);
				} else {
					$this->failed();
				}
			} else {
				$this->failed();
			}

		}

		public function stories() {

			if($this->get_request_method() != "GET") $this->response('',406);
			$limit = isset($this->_request['count']) ? ((int)$this->_request['count']) : 10;
			$page = isset($this->_request['page']) ? ((int)$this->_request['page']) : 1;

			$book_id = isset($this->_request['book_id']) ? ((int)$this->_request['book_id']) : 0;

			$sort = isset($this->_request['sort']) ? ($this->_request['sort']) : 'story_id';
			$order = isset($this->_request['order']) ? ($this->_request['order']) : 'ASC';

			$settings = $this->get_one_result("SELECT api_key FROM tbl_settings WHERE id = '1'");
			$api_key = implode(" ", $settings);

			$offset = ($page * $limit) - $limit;
			$count_total = $this->get_count_result("SELECT COUNT(DISTINCT story_id) FROM tbl_story s, tbl_book b WHERE s.book_id = b.book_id AND s.book_id = $book_id");

			$sql = "SELECT DISTINCT s.*, b.book_name FROM tbl_story s, tbl_book b WHERE s.book_id = b.book_id AND s.book_id = $book_id ORDER BY $sort $order LIMIT $limit OFFSET $offset";

			$posts = $this->get_list_result($sql);
			$count = count($posts);

			$respon = array(
				'status' => 'ok',
				'count' => $count,
				'count_total' => $count_total,
				'pages' => $page,
				'stories' => $posts
			);

			if (isset($_GET['api_key'])) {
				$key_received = $_GET['api_key'];
				if ($key_received == $api_key) {
					$this->response($this->json($respon), 200);
				} else {
					$this->failed();
				}
			} else {
				$this->failed();
			}

		}

		public function search() {

			if($this->get_request_method() != "GET") $this->response('',406);
			$limit = isset($this->_request['count']) ? ((int)$this->_request['count']) : 10;
			$page = isset($this->_request['page']) ? ((int)$this->_request['page']) : 1;

			$q = $_GET['q'];

			$settings = $this->get_one_result("SELECT api_key FROM tbl_settings WHERE id = '1'");
			$api_key = implode(" ", $settings);

			$offset = ($page * $limit) - $limit;
			$count_total = $this->get_count_result("SELECT COUNT(DISTINCT book_id) FROM tbl_book WHERE (book_name LIKE N'%$q%' OR author LIKE N'%$q%')");

			$sql = "SELECT b.*, c.category_name FROM tbl_book b, tbl_category c WHERE b.category_id = c.cid AND (b.book_name LIKE N'%$q%' OR b.author LIKE N'%$q%') LIMIT $limit OFFSET $offset";

			$posts = $this->get_list_result($sql);
			$count = count($posts);

			$respon = array(
				'status' => 'ok',
				'count' => $count,
				'count_total' => $count_total,
				'pages' => $page,
				'books' => $posts
			);

			if (isset($_GET['api_key'])) {
				$key_received = $_GET['api_key'];
				if ($key_received == $api_key) {
					$this->response($this->json($respon), 200);
				} else {
					$this->failed();
				}
			} else {
				$this->failed();
			}

		}

		public function settings() {

			if($this->get_request_method() != "GET") $this->response('',406);
			$limit = isset($this->_request['count']) ? ((int)$this->_request['count']) : 10;
			$page = isset($this->_request['page']) ? ((int)$this->_request['page']) : 1;

			$settings = $this->get_one_result("SELECT api_key FROM tbl_settings WHERE id = '1'");
			$api_key = implode(" ", $settings);

			$sql_settings = "SELECT * FROM tbl_settings";
			$sql_ads = "SELECT * FROM tbl_ads";

			$settings = $this->get_one_result($sql_settings);
			$ads = $this->get_one_result($sql_ads);

			$respon = array(
				'status' => 'ok',
				'setting' => $settings,
				'ads' => $ads
			);

			if (isset($_GET['api_key'])) {
				$key_received = $_GET['api_key'];
				if ($key_received == $api_key) {
					$this->response($this->json($respon), 200);
				} else {
					$this->failed();
				}
			} else {
				$this->failed();
			}

		}

		public function update_view() {

			include "../../includes/config.php";

			if ($_SERVER['REQUEST_METHOD'] == 'POST') {

				$response = array();

				$book_id = $_POST['book_id'];

				$sql = "UPDATE tbl_book SET view_count = view_count + 1 WHERE book_id = '$book_id'";

				if(mysqli_query($connect, $sql)) {
					$response["value"] = 1;
					$response["message"] = "success update views";

					header( 'Content-Type: application/json; charset=utf-8' );
					echo json_encode($response);
				} else {
					$response["value"] = 0;
					$response["message"] = "failed update vieww!";

					header( 'Content-Type: application/json; charset=utf-8' );
					echo json_encode($response);
				}

				mysqli_close($connect);

			}

		}		

		//don't edit all the code below
		private function get_list($query) {
			$r = $this->mysqli->query($query) or die($this->mysqli->error.__LINE__);
			if($r->num_rows > 0) {
				$result = array();
				while($row = $r->fetch_assoc()) {
					$result[] = $row;
				}
				$this->response($this->json($result), 200); // send user details
			}
			$this->response('',204);	// If no records "No Content" status
		}

		private function get_list_result($query) {
			$result = array();
			$r = $this->mysqli->query($query) or die($this->mysqli->error.__LINE__);
			if($r->num_rows > 0) {
				while($row = $r->fetch_assoc()) {
					$result[] = $row;
				}
			}
			return $result;
		}

		private function get_object_result($query) {
			$result = array();
			$r = $this->mysqli->query($query) or die($this->mysqli->error.__LINE__);
			if($r->num_rows > 0) {
				while($row = $r->fetch_assoc()) {
					$result = $row;
				}
			}
			return $result;
		}

		private function get_category_result($query) {
			$result = array();
			$r = $this->mysqli->query($query) or die($this->mysqli->error.__LINE__);
			if($r->num_rows > 0) {
				while($row = $r->fetch_assoc()) {
					$result = $row;
				}
			}
			return $result;
		}

		private function get_one_result($query) {
				$result = array();
				$r = $this->mysqli->query($query) or die($this->mysqli->error.__LINE__);
				if($r->num_rows > 0) $result = $r->fetch_assoc();
				return $result;
		}

		private function get_one($query) {
			$r = $this->mysqli->query($query) or die($this->mysqli->error.__LINE__);
			if($r->num_rows > 0) {
				$result = $r->fetch_assoc();
				$this->response($this->json($result), 200); // send user details
			}
			$this->response('',204);	// If no records "No Content" status
		}

		private function get_count($query) {
			$r = $this->mysqli->query($query) or die($this->mysqli->error.__LINE__);
			if($r->num_rows > 0) {
				$result = $r->fetch_row();
				$this->response($result[0], 200);
			}
			$this->response('',204);	// If no records "No Content" status
		}

		private function get_count_result($query) {
			$r = $this->mysqli->query($query) or die($this->mysqli->error.__LINE__);
			if($r->num_rows > 0) {
				$result = $r->fetch_row();
				return $result[0];
			}
			return 0;
		}

		private function post_one($obj, $column_names, $table_name) {
			$keys 		= array_keys($obj);
			$columns 	= '';
			$values 	= '';
			foreach($column_names as $desired_key) { // Check the recipe received. If blank insert blank into the array.
			  if(!in_array($desired_key, $keys)) {
			   	$$desired_key = '';
				} else {
					$$desired_key = $obj[$desired_key];
				}
				$columns 	= $columns.$desired_key.',';
				$values 	= $values."'".$this->real_escape($$desired_key)."',";
			}
			$query = "INSERT INTO ".$table_name."(".trim($columns,',').") VALUES(".trim($values,',').")";
			//echo "QUERY : ".$query;
			if(!empty($obj)) {
				//$r = $this->mysqli->query($query) or trigger_error($this->mysqli->error.__LINE__);
				if ($this->mysqli->query($query)) {
					$status = "success";
			    $msg 		= $table_name." created successfully";
				} else {
					$status = "failed";
			    $msg 		= $this->mysqli->error.__LINE__;
				}
				$resp = array('status' => $status, "msg" => $msg, "data" => $obj);
				$this->response($this->json($resp),200);
			} else {
				$this->response('',204);	//"No Content" status
			}
		}

		private function post_update($id, $obj, $column_names, $table_name) {
			$keys = array_keys($obj[$table_name]);
			$columns = '';
			$values = '';
			foreach($column_names as $desired_key){ // Check the recipe received. If key does not exist, insert blank into the array.
			  if(!in_array($desired_key, $keys)) {
			   	$$desired_key = '';
				} else {
					$$desired_key = $obj[$table_name][$desired_key];
				}
				$columns = $columns.$desired_key."='".$this->real_escape($$desired_key)."',";
			}

			$query = "UPDATE ".$table_name." SET ".trim($columns,',')." WHERE id=$id";
			if(!empty($obj)) {
				// $r = $this->mysqli->query($query) or die($this->mysqli->error.__LINE__);
				if ($this->mysqli->query($query)) {
					$status = "success";
					$msg 	= $table_name." update successfully";
				} else {
					$status = "failed";
					$msg 	= $this->mysqli->error.__LINE__;
				}
				$resp = array('status' => $status, "msg" => $msg, "data" => $obj);
				$this->response($this->json($resp),200);
			} else {
				$this->response('',204);	// "No Content" status
			}
		}

		private function delete_one($id, $table_name) {
			if($id > 0) {
				$query="DELETE FROM ".$table_name." WHERE id = $id";
				if ($this->mysqli->query($query)) {
					$status = "success";
			    $msg 		= "One record " .$table_name." successfully deleted";
				} else {
					$status = "failed";
			    $msg 		= $this->mysqli->error.__LINE__;
				}
				$resp = array('status' => $status, "msg" => $msg);
				$this->response($this->json($resp),200);
			} else {
				$this->response('',204);	// If no records "No Content" status
			}
		}

		private function responseInvalidParam() {
			$resp = array("status" => 'Failed', "msg" => 'Invalid Parameter' );
			$this->response($this->json($resp), 200);
		}

		private function failed() {
			$resp = array('status' => 'error', 'code' => '403', 'message' => 'The request is missing a valid API key.');
			$this->response($this->json($resp), 200);
			exit;
		}

		/* ==================================== End of API utilities ==========================================
		 * ====================================================================================================
		 */

		/* Encode array into JSON */
		private function json($data) {
			if(is_array($data)) {
				return json_encode($data, JSON_NUMERIC_CHECK);
			}
		}

		/* String mysqli_real_escape_string */
		private function real_escape($s) {
			return mysqli_real_escape_string($this->mysqli, $s);
		}
	}

	// Initiate Library
	$api = new API;
	if (isset($_GET['check_connection'])) {
		$api->check_connection();
	} else if (isset($_GET['categories'])) {
		$api->categories();
	} else if (isset($_GET['books'])) {
		$api->books();
	} else if (isset($_GET['book_details'])) {
		$api->book_details();
	} else if (isset($_GET['category_details'])) {
		$api->category_details();
	} else if (isset($_GET['stories'])) {
		$api->stories();
	} else if (isset($_GET['search'])) {
		$api->search();
	} else if (isset($_GET['settings'])) {
		$api->settings();
	} else if (isset($_GET['update_view'])) {
		$api->update_view();
	} else {
		$api->processApi();
	}
?>
